﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using converter;

namespace converter.test
{
    [TestClass]
    public class TestCommandLine
    {
        [TestMethod]
        public void SingleRdl()
        {
            string[] args = { "-rdl:report.rdl" };
            CommandLine commandLine = new CommandLine(args);

            Assert.AreEqual<int>(1, commandLine.Reports.Count);
            Assert.AreEqual<string>("report.docx", commandLine.Reports["report.rdl"]);
        }

        [TestMethod]
        public void SingleRdlSingleOut()
        {
            string[] args = { "-rdl:report.rdl", "-out:template.docx" };
            CommandLine commandLine = new CommandLine(args);

            Assert.AreEqual<int>(1, commandLine.Reports.Count);
            Assert.AreEqual<string>("template.docx", commandLine.Reports["report.rdl"]);
        }

        [TestMethod]
        public void SingleRdlMultipleOut()
        {
            string[] args = { "-rdl:report1.rdl", "-out:template1.docx", "-out:template2.docx" };
            CommandLine commandLine = new CommandLine(args);

            Assert.AreEqual<int>(1, commandLine.Reports.Count);
            Assert.AreEqual<string>("report1.docx", commandLine.Reports["report1.rdl"]);
        }

        [TestMethod]
        public void MultipleRdl()
        {
            string[] args = { "-rdl:report1.rdl", "-rdl:report2.rdlc" };
            CommandLine commandLine = new CommandLine(args);

            Assert.AreEqual<int>(2, commandLine.Reports.Count);
            Assert.AreEqual<string>("report1.docx", commandLine.Reports["report1.rdl"]);
            Assert.AreEqual<string>("report2.docx", commandLine.Reports["report2.rdlc"]);
        }

        [TestMethod]
        public void MultipleRdlSingleOut()
        {
            string[] args = { "-rdl:report1.rdl", "-rdl:report2.rdlc", "-out:template.docx" };
            CommandLine commandLine = new CommandLine(args);

            Assert.AreEqual<int>(2, commandLine.Reports.Count);
            Assert.AreEqual<string>("report1.docx", commandLine.Reports["report1.rdl"]);
            Assert.AreEqual<string>("report2.docx", commandLine.Reports["report2.rdlc"]);
        }

        [TestMethod]
        public void MultipleRdlMultipleOut()
        {
            string[] args = { "-rdl:report1.rdl", "-rdl:report2.rdlc", "-out:template1.docx", "-out:template2.docx" };
            CommandLine commandLine = new CommandLine(args);

            Assert.AreEqual<int>(2, commandLine.Reports.Count);
            Assert.AreEqual<string>("report1.docx", commandLine.Reports["report1.rdl"]);
            Assert.AreEqual<string>("report2.docx", commandLine.Reports["report2.rdlc"]);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException), "Command line must contain at least one RDL file.")]
        public void EmptyArguments()
        {
            string[] args = new string[0];
            new CommandLine(args);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void NullArguments()
        {
            new CommandLine(null);
        }

        [TestMethod]
        public void ArgumentsInMixedCase()
        {
            string[] args = { "-RDL:report.rdl", "-rDl:report2.rdl", "-rdl:REPORT3.rdl" };
            CommandLine commandLine = new CommandLine(args);

            Assert.AreEqual<int>(3, commandLine.Reports.Count);
            Assert.AreEqual<string>("report.docx", commandLine.Reports["report.rdl"]);
            Assert.AreEqual<string>("report2.docx", commandLine.Reports["report2.rdl"]);
            Assert.AreEqual<string>("REPORT3.docx", commandLine.Reports["report3.rdl"]);
        }

        [TestMethod]
        public void EqualArguments()
        {
            string[] args = { "-rdl:report.rdl", "-rdl:report.rdl", "-rdl:REPORT.rdl" };
            CommandLine commandLine = new CommandLine(args);

            Assert.AreEqual<int>(1, commandLine.Reports.Count);
            Assert.AreEqual<string>("report.docx", commandLine.Reports["report.rdl"]);
        }

        [TestMethod]
        public void ArgumentsStartWithDashOrSlash()
        {
            string[] args = { "/rdl:report1.rdl", "-rdl:report2.rdl" };
            CommandLine commandLine = new CommandLine(args);

            Assert.AreEqual<int>(2, commandLine.Reports.Count);
            Assert.AreEqual<string>("report1.docx", commandLine.Reports["report1.rdl"]);
            Assert.AreEqual<string>("report2.docx", commandLine.Reports["report2.rdl"]);
        }

        [TestMethod]
        public void BadArguments()
        {
            string[] args = { "opt", "-opt", "/rdl", "/rdl:", "/out:", "-rdl:report.rdl", "-:", "-", ":" };
            CommandLine commandLine = new CommandLine(args);

            Assert.AreEqual<int>(1, commandLine.Reports.Count);
            Assert.AreEqual<string>("report.docx", commandLine.Reports["report.rdl"]);
        }

        [TestMethod]
        public void OverwriteIsFalseByDefault()
        {
            string[] args = { "-rdl:report.rdl" };
            CommandLine commandLine = new CommandLine(args);

            Assert.AreEqual<bool>(false, commandLine.Overwrite);
        }

        [TestMethod]
        public void TestOverwrite()
        {
            string[] args = { "-rdl:report.rdl", "/y" };
            CommandLine commandLine = new CommandLine(args);

            Assert.AreEqual<bool>(true, commandLine.Overwrite);
        }

        [TestMethod]
        public void TestDataSources()
        {
            string[] args = { "-rdl:report.rdl", "-rds:data.rds", "-rds:data2.rds", "-rds:data2.rds", "-rds:DATA2.rds" };
            CommandLine commandLine = new CommandLine(args);

            Assert.AreEqual<int>(2, commandLine.DataSources.Count);
            Assert.AreEqual<string>("data.rds", commandLine.DataSources[0]);
            Assert.AreEqual<string>("data2.rds", commandLine.DataSources[1]);
        }
    }
}
